-- ----------------
-- CLOB-SPEICHERUNG
-- ----------------

--- Tabelle anlegen

DROP TABLE GemeindePOIs;

CREATE TABLE GemeindePOIs (
  gkz       INTEGER,                    -- Gemeindeschlssel
  poi_xml   XMLTYPE,                    -- POIs in XML
  CONSTRAINT pk_gpoi PRIMARY KEY (gkz), -- Definition Primrschlssel
  CONSTRAINT fk_gpoi_gkz FOREIGN KEY (gkz) REFERENCES Gemeinden(gkz) 
                                        -- Fremdschlssel
);

--- Daten einfgen

INSERT INTO GemeindePOIs (gkz) VALUES (3403000);

UPDATE GemeindePOIs SET poi_xml = XMLTYPE
('<?xml version="1.0"?>
  <poi-set>
    <poi>
      <name>Schloss</name>
      <Point> <pos>8.21271 53.13930</pos> </Point>
    </poi>
    <poi>
      <name>FH</name>
      <Point> <pos>8.19974 53.14253</pos> </Point>
    </poi>
    <poi>
      <name>Bahnhof</name>
      <Point> <pos>8.22025 53.14416</pos> </Point>
    </poi>
  </poi-set>'
) WHERE gkz = 3403000;

COMMIT;

SET LONG 2000

-- XML abfragen:
SELECT gkz, poi_xml FROM GemeindePOIs;

--- XML ndern

-- Textknoten ndern:
UPDATE GemeindePOIs
SET poi_xml = updateXML(poi_xml,'//poi[1]/name/text()','Schloss')
WHERE gkz = 3403000;

-- Element ersetzen:
UPDATE GemeindePOIs
SET poi_xml = updateXML(poi_xml,'//poi[2]/name','<name>FH Oldenburg</name>')
WHERE gkz = 3403000;

COMMIT;

-- XML abfragen:
SELECT poi_xml FROM GemeindePOIs WHERE gkz = 3403000;


-- -------------------------
-- STRUKTURIERTE SPEICHERUNG
-- -------------------------

-- Zunchst muss das XML-Schema (z.B. ber den Oracle Enterprise Manager) in die Datenbank geladen werden.

--- Tabelle anlegen

DROP TABLE GemeindePOIsStr;

CREATE TABLE GemeindePOIsStr (
  gkz       INTEGER,                     -- Gemeindeschlssel
  poi_xml   XMLTYPE,                     -- POIs in XML
  CONSTRAINT pk_gpois PRIMARY KEY (gkz), -- Definition Primrschlssel
  CONSTRAINT fk_gpois_gkz FOREIGN KEY (gkz) REFERENCES Gemeinden(gkz) 
                                         -- Fremdschlssel
)
XMLTYPE COLUMN poi_xml
XMLSCHEMA "http://www.geodbs.de/xml/gemeindepoi.xsd"
ELEMENT "poi-set";

--- Fehlerhaftes Einfgen

INSERT INTO GemeindePOIsStr (gkz,poi_xml) VALUES (3403000,XMLTYPE
('<?xml version="1.0"?>
  <poi-set xmlns="http://www.geodbs.de/xml/gemeindepoi"
           xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
           xsi:schemaLocation="http://www.geodbs.de/xml/gemeindepoi http://www.geodbs.de/xml/gemeindepoi.xsd">
    <gem-poi>
      <name>Schloss</name>
    </gem-poi>
  </poi-set>'));

--- Zunchst erfolgreiches Einfgen

INSERT INTO GemeindePOIsStr (gkz,poi_xml) VALUES (3403000,XMLTYPE
('<?xml version="1.0"?>
  <poi-set xmlns="http://www.geodbs.de/xml/gemeindepoi"
           xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
           xsi:schemaLocation="http://www.geodbs.de/xml/gemeindepoi http://www.geodbs.de/xml/gemeindepoi.xsd">
    <poi>
      <name>Schloss</name>
    </poi>
  </poi-set>'));

ROLLBACK;

--- vollstndige Validierung

-- Integrittsbedingung hinzufgen:
ALTER TABLE GemeindePOIsStr
ADD CONSTRAINT valid_gempoi CHECK (XMLIsValid(poi_xml)=1);

-- Ungltige Daten einfgen:
INSERT INTO GemeindePOIsStr (gkz,poi_xml) VALUES (3403000,XMLTYPE
('<?xml version="1.0"?>
  <poi-set xmlns="http://www.geodbs.de/xml/gemeindepoi"
           xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
           xsi:schemaLocation="http://www.geodbs.de/xml/gemeindepoi http://www.geodbs.de/xml/gemeindepoi.xsd">
    <poi>
      <name>Schloss</name>
    </poi>
  </poi-set>'));

--- Erfolgreiches Einfgen

INSERT INTO GemeindePOIsStr (gkz,poi_xml) VALUES (3403000,XMLTYPE
('<?xml version="1.0"?>
  <poi-set xmlns="http://www.geodbs.de/xml/gemeindepoi"
           xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
           xsi:schemaLocation="http://www.geodbs.de/xml/gemeindepoi http://www.geodbs.de/xml/gemeindepoi.xsd">
    <poi>
      <name>Schloss</name>
      <Point> <pos>8.21271 53.13930</pos> </Point>
    </poi>
    <poi>
      <name>FH</name>
      <Point> <pos>8.19974 53.14253</pos> </Point>
    </poi>
    <poi>
      <name>Bahnhof</name>
      <Point> <pos>8.22025 53.14416</pos> </Point>
    </poi>
  </poi-set>'));

COMMIT;


-- --------------
-- XPath-ANFRAGEN
-- --------------

SET LONG 2000

--- existNode

-- Aufruf als Methode:
SELECT gkz FROM GemeindePOIs g
WHERE g.poi_xml.existsNode('//Point[./pos/text()="8.21271 53.13930"]') = 1;

-- Aufruf als Funktion:
SELECT existsNode(poi_xml,'//poi/pos') FROM GemeindePOIs;

--- extract

-- Aufruf als Methode:
SELECT g.poi_xml.extract('//Point') FROM GemeindePOIs g;

-- Aufruf als Funktion:
SELECT COUNT(extract(poi_xml,'//Point')) FROM GemeindePOIs;

-- Nutzung von getStringVal():
SELECT gkz FROM GemeindePOIs g
WHERE g.poi_xml.extract('/poi-set/poi[3]/name/text()').getStringVal() = 'Bahnhof';

SELECT value(t)
FROM GemeindePOIs g, 
     TABLE (XMLSequence (g.poi_xml.extract('//Point'))) t;

SELECT COUNT(value(t))
FROM GemeindePOIs g, 
     TABLE (XMLSequence (g.poi_xml.extract('//Point'))) t;

--- extractValue

SELECT extract(poi_xml,'//pos/text()[../../../name/text()="Bahnhof"]')
FROM GemeindePOIs;

SELECT extractValue(poi_xml,'//pos[../../name/text()="Bahnhof"]')
FROM GemeindePOIs;

SELECT extractValue(value(t),'//pos')
FROM GemeindePOIs g, TABLE (xmlsequence (g.poi_xml.extract('//Point'))) t;


-- ---------------------------------
-- XSL-STYLESHEET und TRANSFORMATION
-- ---------------------------------

-- Tabelle angelegen

DROP TABLE XSL;

CREATE TABLE XSL (
   name       VARCHAR(80),              -- Name der Transformation
   stylesheet XMLTYPE,                  -- die Transformation
   CONSTRAINT pk_xsl PRIMARY KEY (name) -- Definition Primrschlssel
);

-- Transformation

-- Transformation speichern:
INSERT INTO XSL VALUES ('gml2svg', XMLTYPE(
'<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
   <xsl:output method="xml" indent="yes" media-type="text/xml"/>
   <xsl:variable name="rad">5</xsl:variable>
   <xsl:variable name="mult">10000</xsl:variable>
   <xsl:variable name="dx">81950</xsl:variable>
   <xsl:variable name="dy">531500</xsl:variable>
   <xsl:template match="/poi-set">
     <svg width="350" height="200">
       <rect x="0" y="0" width="350" height="200" fill="lightgreen" stroke="black"/>
       <xsl:apply-templates select="poi"/>
     </svg>
   </xsl:template>
   <xsl:template match="poi">
     <xsl:call-template name="shapes">
       <xsl:with-param name="xc" select="substring(Point/pos,0,7)" />
       <xsl:with-param name="yc" select="substring(Point/pos,8,8)" />
       <xsl:with-param name="txt" select="name" />
     </xsl:call-template>
   </xsl:template>
   <xsl:template name="shapes">
     <xsl:param name="xc" />
     <xsl:param name="yc" />
     <xsl:param name="txt" />
     <xsl:element name="circle">
       <xsl:attribute name="cx"><xsl:value-of select="(number($xc)*$mult)-$dx"/></xsl:attribute>
       <xsl:attribute name="cy"><xsl:value-of select="number($dy)-(number($yc)*$mult)"/></xsl:attribute>
       <xsl:attribute name="r"><xsl:value-of select="$rad"/></xsl:attribute>
       <xsl:attribute name="fill">red</xsl:attribute>
       <xsl:attribute name="stroke">black</xsl:attribute>
     </xsl:element>
     <xsl:element name="text">
       <xsl:attribute name="x"><xsl:value-of select="(number($xc)*$mult)-$dx+$rad"/></xsl:attribute>
       <xsl:attribute name="y"><xsl:value-of select="number($dy)-(number($yc)*$mult)"/></xsl:attribute>
       <xsl:value-of select="$txt"/>
     </xsl:element>
   </xsl:template>
 </xsl:stylesheet>') );
 
COMMIT;

-- Transformation ausfhren:
SELECT XMLTransform (g.poi_xml,sheet.stylesheet)
FROM GemeindePOIs g, XSL sheet
WHERE gkz = 3403000 AND sheet.name = 'gml2svg';


-- -------------
-- GML-ERZEUGUNG
-- -------------

-- TO_GMLGEOMETRY

SELECT name, TO_CHAR(SDO_UTIL.TO_GMLGEOMETRY(geo))
FROM GeoDbLand
WHERE id = 2 OR id = 4;

-- Java fr GML-Erzeugung

-- von KOMMANDOZEILE aus Java laden:
loadjava -user <Benutzer>/<Passwort>@<Dienstname> -andresolve GenerateGML.java

-- Java-Methode bekanntgeben:
CREATE OR REPLACE FUNCTION gml (geom MDSYS.SDO_GEOMETRY) RETURN VARCHAR2 AS
   LANGUAGE JAVA NAME 'GenerateGML.toGML (oracle.sql.STRUCT)
                                                  return java.lang.String';
/

-- Methode nutzen:
SELECT name, gml(geo) FROM GeoDbLand;


-- ------------
-- GML-ANFRAGEN
-- ------------

--- Positionstabelle anlegen:
CREATE TABLE POIPositionen (
  gkz   INTEGER,              -- Gemeindekennzeichen
  name  VARCHAR(40),          -- POI-Name
  pos   MDSYS.SDO_GEOMETRY,   -- Position
  CONSTRAINT fk_pospoi_gkz FOREIGN KEY (gkz)  -- Fremdschlssel 
     REFERENCES GemeindePOIs(gkz) ON DELETE CASCADE
);

--- Metadaten bekanntgeben:
INSERT INTO USER_SDO_GEOM_METADATA
VALUES ('POIPositionen','pos', MDSYS.SDO_DIM_ARRAY(
           MDSYS.SDO_DIM_ELEMENT('Lnge', -180,180, 10),
           MDSYS.SDO_DIM_ELEMENT('Breite', -90,90, 10)), 8307);
COMMIT;

--- Index anlegen:
CREATE INDEX POIPositionen_pos_ix ON POIPositionen(pos)
INDEXTYPE IS MDSYS.SPATIAL_INDEX PARAMETERS('LAYER_GTYPE=POINT');

-- von KOMMANDOZEILE aus Java laden:
loadjava -user <Benutzer>/<Passwort>@<Dienstname> -andresolve ProcessGML.java

-- Java-Methode bekannt geben:
CREATE OR REPLACE PROCEDURE processXML
   (gkz NUMBER, name VARCHAR, coords VARCHAR) AS LANGUAGE JAVA NAME
      'ProcessGML.insertPos(int,java.lang.String,java.lang.String)';
/

-- Trigger erzeugen:
CREATE OR REPLACE TRIGGER gml_trigger
AFTER INSERT OR UPDATE ON GemeindePOIs FOR EACH ROW
DECLARE
  CURSOR c IS SELECT extractValue(value(t),'/poi/name') name,
                     extractValue(value(t),'/poi/Point/pos') coords
              FROM TABLE (xmlsequence (:new.poi_xml.extract('//poi'))) t;
  c_val c%ROWTYPE;
BEGIN
  DELETE FROM POIPositionen WHERE gkz = :old.gkz;
  OPEN c;
  LOOP
    FETCH c INTO c_val;
    EXIT WHEN c%NOTFOUND;
    processXML(:new.gkz,c_val.name,c_val.coords);
  END LOOP;
  CLOSE c;
END;
/

-- POIs aktualisieren:
UPDATE GemeindePOIs SET poi_xml = XMLTYPE
('<?xml version="1.0"?>
  <poi-set>
    <poi>
      <name>Schloss</name>
      <Point> <pos>8.21271 53.13930</pos> </Point>
    </poi>
    <poi>
      <name>FH Oldenburg</name>
      <Point> <pos>8.19974 53.14253</pos> </Point>
    </poi>
    <poi>
      <name>Bahnhof</name>
      <Point> <pos>8.22025 53.14416</pos> </Point>
    </poi>
  </poi-set>')
WHERE gkz = 3403000;
COMMIT;

-- Resultat berprfen:
SELECT name, pos FROM POIPositionen;

-- Rumliche Anfrage
SELECT g.poi_xml.extract('//poi[./name/text()="' || r.name || '"]')
FROM GemeindePOIs g INNER JOIN 
  (SELECT p.gkz,name
   FROM POIPositionen p
   WHERE SDO_RELATE (pos, SDO_CS.VIEWPORT_TRANSFORM(
                           MDSYS.SDO_GEOMETRY(2003,0,null,
                             MDSYS.SDO_ELEM_INFO_ARRAY(1,1003,3),
                             MDSYS.SDO_ORDINATE_ARRAY(8.20,53, 9,54)), 8307),
                     'mask=ANYINTERACT') = 'TRUE')  r
ON g.gkz = r.gkz;
