-- -------------------
-- Klasse SDO_GEOMETRY
-- -------------------

-- Anlegen der Tabelle "GeoDbLand"

CREATE TABLE GeoDbLand (
  id     INTEGER,                        -- Identifikationsnummer
  name   VARCHAR(20),                    -- Objektname
  geo    MDSYS.SDO_GEOMETRY,             -- Objektgeometrie
  CONSTRAINT pk_gdbland PRIMARY KEY(id)  -- Primrschlssel
);

-- Einfgen von Daten

INSERT INTO GeoDbLand (id,name,geo)
VALUES (1,'Flaggenmast',MDSYS.SDO_GEOMETRY(2001,NULL,MDSYS.SDO_POINT_TYPE(10,11,NULL),NULL,NULL));
COMMIT;

SELECT * FROM GeoDbLand;

UPDATE GeoDbLand
SET geo=MDSYS.SDO_GEOMETRY(2001,NULL,NULL,MDSYS.SDO_ELEM_INFO_ARRAY(1,1,1),MDSYS.SDO_ORDINATE_ARRAY(10,11))
WHERE id = 1;
COMMIT;

INSERT INTO GeoDbLand (id,name,geo)
VALUES (2,'Steg',MDSYS.SDO_GEOMETRY(2002,NULL,NULL,MDSYS.SDO_ELEM_INFO_ARRAY(1,2,1),MDSYS.SDO_ORDINATE_ARRAY(4,4,5,7,8,8)));
INSERT INTO GeoDbLand (id,name,geo)
VALUES (3,'Baumreihe',MDSYS.SDO_GEOMETRY(2005,NULL,NULL,MDSYS.SDO_ELEM_INFO_ARRAY(1,1,3),MDSYS.SDO_ORDINATE_ARRAY(10,19,12,19,14,19)));
INSERT INTO GeoDbLand (id,name,geo)
VALUES (4,'Wall',MDSYS.SDO_GEOMETRY(2002,NULL,NULL,MDSYS.SDO_ELEM_INFO_ARRAY(1,2,2),MDSYS.SDO_ORDINATE_ARRAY(1,15,2.414,15.586,3,17,3.586,18.414,5,19)));
COMMIT;

INSERT INTO GeoDbLand (id,name,geo)
VALUES (5,'Haus',MDSYS.SDO_GEOMETRY(2003,NULL,NULL,MDSYS.SDO_ELEM_INFO_ARRAY(1,1003,3),MDSYS.SDO_ORDINATE_ARRAY(9,9,12,10)));
COMMIT;

UPDATE GeoDbLand
SET geo=MDSYS.SDO_GEOMETRY(2003,NULL,NULL,MDSYS.SDO_ELEM_INFO_ARRAY(1,1003,1),MDSYS.SDO_ORDINATE_ARRAY(9,9,12,9,12,10,9,10,9,9))
WHERE id = 5;
COMMIT;

INSERT INTO GeoDbLand (id,name,geo)
VALUES (6,'Land',MDSYS.SDO_GEOMETRY(2007,NULL,NULL,MDSYS.SDO_ELEM_INFO_ARRAY(1,1003,1,11,2003,1,29,1003,1),
MDSYS.SDO_ORDINATE_ARRAY(1,1,19,1,19,20,1,20,1,1,4,5,3,10,7,14,13,15,17,12,16,8,13,5,9,3,4,5,7,7,12,7,13,11,10,13,7,10,7,7)));
COMMIT;

UPDATE GeoDbLand
SET geo=MDSYS.SDO_GEOMETRY(2002,NULL,NULL,MDSYS.SDO_ELEM_INFO_ARRAY(1,4,2,1,2,2,9,2,1),MDSYS.SDO_ORDINATE_ARRAY(1,15,2.414,15.586,3,17,3.586,18.414,5,19,7,19))
WHERE name = 'Wall';
COMMIT;

-- WKT-Konstruktor:
UPDATE GeoDbLand
SET geo = SDO_GEOMETRY('POINT(10 11)')
WHERE id = 1;
COMMIT;

-- Daten anzeigen:
SELECT * FROM GeoDbLand;


-- ---------
-- METADATEN
-- ---------

INSERT INTO USER_SDO_GEOM_METADATA
VALUES ('GEODBLAND','GEO', MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X',0,20,0.001),MDSYS.SDO_DIM_ELEMENT('Y',0,20,0.001)) ,NULL);
COMMIT;


-- --------
-- METHODEN
-- --------

SELECT name,g.geo.Get_Dims() FROM GeoDbLand g;

SELECT name, g.geo.Get_GType() FROM GeoDbLand g;

SELECT name, g.geo.Get_LRS_Dim() FROM GeoDbLand g;

-- Objektmethoden ab V10.0:

SELECT name, g.geo.ST_CoordDim() FROM GeoDbLand g;

SET LONG 1000

SELECT g.geo.Get_WKT()
FROM GeoDbLand g
WHERE name <> 'Wall';


-- -----------------------
-- GEOMETRISCHE FUNKTIONEN
-- -----------------------

--- Geometrieprfung

-- Fehlerhafte Geometrie einfgen:
INSERT INTO GeoDbLand (id,name,geo)
VALUES (7,'Haus 2',MDSYS.SDO_GEOMETRY(2003,NULL,NULL,MDSYS.SDO_ELEM_INFO_ARRAY(1,1003,1),MDSYS.SDO_ORDINATE_ARRAY(9,9,9,10,12,10,12,9,9,9)));

SELECT id FROM GeoDbLand WHERE SDO_GEOM.VALIDATE_GEOMETRY_WITH_CONTEXT(geo,0.001) <> 'TRUE';

SELECT id,SDO_GEOM.VALIDATE_GEOMETRY_WITH_CONTEXT(geo,m.diminfo)
FROM GeoDbLand,USER_SDO_GEOM_METADATA m
WHERE m.table_name = 'GEODBLAND' AND m.column_name = 'GEO';

-- ab 10.1 auch:
SELECT id, g.geo.ST_IsValid() FROM GeoDbLand g;

-- Geometrie korrigieren:
UPDATE GeoDbLand
SET geo = SDO_MIGRATE.TO_CURRENT (geo,
  MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('X',0,20,0.001),
                      MDSYS.SDO_DIM_ELEMENT('Y',0,20,0.001)) )
WHERE name = 'Haus 2';

-- Erneute Prfung:
SELECT id,SDO_GEOM.VALIDATE_GEOMETRY_WITH_CONTEXT(geo,m.diminfo)
FROM GeoDbLand,USER_SDO_GEOM_METADATA m
WHERE m.table_name = 'GEODBLAND' AND m.column_name = 'GEO';

ROLLBACK;

-- Duplikate entfernen

SELECT geo, SDO_UTIL.REMOVE_DUPLICATE_VERTICES(geo,4)
FROM GeoDbLand
WHERE name = 'Land';

--- Topologische Beziehungen

SELECT SDO_GEOM.RELATE(g1.geo,'ANYINTERACT',g2.geo,0.001)
FROM GeoDbLand g1, GeoDbLand g2
WHERE g1.name = 'Haus' AND g2.name = 'Land';

SELECT SDO_GEOM.RELATE(g1.geo,'INSIDE+EQUAL',g2.geo,0.001)
FROM GeoDbLand g1, GeoDbLand g2
WHERE g1.name = 'Haus' AND g2.name = 'Land';

SELECT SDO_GEOM.RELATE(g1.geo,'TOUCH',g2.geo,0.001)
FROM GeoDbLand g1, GeoDbLand g2
WHERE g1.name = 'Haus' AND g2.name = 'Land';

SELECT SDO_GEOM.RELATE(g1.geo,'DETERMINE',g2.geo,0.001)
FROM GeoDbLand g1, GeoDbLand g2
WHERE g1.name = 'Haus' AND g2.name = 'Land';

--- Funktionen, die sich auf ein Geoobjekt beziehen

-- minimal umgebendes Rechteck:
SELECT SDO_GEOM.SDO_MBR(geo) FROM GeoDbLand WHERE name='Wall';

-- Pufferzone:
SELECT SDO_GEOM.SDO_BUFFER(geo,2,0.001) FROM GeoDbLand WHERE name='Haus';

-- Schnitt zwischen Geometrie und Puffer:
SELECT g2.name, SDO_GEOM.RELATE(g2.geo,'ANYINTERACT',SDO_GEOM.SDO_BUFFER(g1.geo,2,0.001),0.001)
FROM GeoDbLand g1 INNER JOIN GeoDbLand g2
ON g1.id <> g2.id
WHERE g1.name = 'Haus';

-- Konvexe Hlle:
SELECT SDO_GEOM.SDO_CONVEXHULL(geo,m.diminfo)
FROM GeoDbLand, MDSYS.USER_SDO_GEOM_METADATA m
WHERE m.table_name = 'GEODBLAND' AND m.column_name = 'GEO' AND name = 'Steg';

-- Lnge:
SELECT name, SDO_GEOM.SDO_LENGTH(geo,0.001) FROM GeoDbLand;

-- Masseinheiten fr Distanzen:
SELECT DISTINCT *
FROM MDSYS.SDO_DIST_UNITS
WHERE conversion_factor >= 0.1 AND conversion_factor <= 1;

-- Flche:
SELECT name, SDO_GEOM.SDO_AREA(geo,0.001)
FROM GeoDbLand g
WHERE g.geo.Get_GType() = 3 OR g.geo.Get_GType() = 7;

-- Masseinheiten fr Flchen:
SELECT DISTINCT *
FROM MDSYS.SDO_AREA_UNITS
WHERE conversion_factor >= 0.1 AND conversion_factor <= 100;

-- Schwerpunkt:
SELECT name, SDO_GEOM.SDO_CENTROID(geo,0.001) FROM GeoDbLand;

-- Punkt von der Flche:
SELECT name, SDO_GEOM.SDO_POINTONSURFACE(geo,0.001) FROM GeoDbLand;

--- Funktionen, die sich auf zwei Geoobjekte beziehen

SELECT g1.name, g2.name, SDO_GEOM.SDO_DISTANCE(g1.geo,g2.geo,0.001)
FROM GeoDbLand g1 INNER JOIN GeoDbLand g2
ON g1.id < g2.id
ORDER BY g1.id, g2.id;

SELECT SDO_GEOM.SDO_DIFFERENCE(g1.geo,g2.geo,0.001)
FROM GeoDbLand g1, GeoDbLand g2
WHERE g1.name = 'Land' AND g2.name = 'Haus';

--- Zugriffs- und Hilfsfunktionen

-- Zugriff auf Punkte:
SELECT id, name, SDO_UTIL.GETVERTICES(geo)
FROM (SELECT * FROM GeoDbLand WHERE id > 1)
WHERE SDO_UTIL.GETNUMVERTICES(geo) = 5;

SELECT g.id, name, t.x, t.y
FROM (SELECT * FROM GeoDbLand WHERE id > 1) g, TABLE(SDO_UTIL.GETVERTICES(g.geo)) t
WHERE SDO_UTIL.GETNUMVERTICES(g.geo) = 5 AND t.id = 3;

-- Zugriff auf Teilgeometrien:
SELECT SDO_UTIL.EXTRACT(geo,1)
FROM GeoDbLand
WHERE Name = 'Land';

SELECT SDO_UTIL.EXTRACT(geo,1,2)
FROM GeoDbLand
WHERE Name = 'Land';

-- Anzahl der Teilgeometrien:
SELECT name,SDO_UTIL.GETNUMELEM(geo)
FROM GeoDbLand
WHERE SDO_UTIL.GETNUMELEM(geo) > 1;

-- Generalisierung:
SELECT SDO_UTIL.SIMPLIFY(geo,2)
FROM GeoDbLand
WHERE Name = 'Land';


-- --------------------------------
-- GEOMETRISCHE AGGREGATSFUNKTIONEN
-- --------------------------------

SELECT COUNT(*), SDO_AGGR_MBR(geo) FROM GeoDbLand;

SELECT SDO_AGGR_CONVEXHULL(MDSYS.SDOAGGRTYPE(geo,0.001))
FROM GeoDbLand g
WHERE g.geo.Get_GType() < 3;

SELECT SDO_AGGR_UNION(MDSYS.SDOAGGRTYPE(geo,0.001))
FROM GeoDbLand;

SELECT g.geo.get_gtype(), SDO_GEOM.SDO_LENGTH(SDO_AGGR_UNION(MDSYS.SDOAGGRTYPE(geo,0.001)),0.001)
FROM GeoDbLand g
GROUP BY g.geo.Get_GType()
HAVING SDO_GEOM.SDO_LENGTH(SDO_AGGR_UNION(MDSYS.SDOAGGRTYPE(geo,0.001)),0.001) > 0;

SELECT SDO_AGGR_CENTROID(MDSYS.SDOAGGRTYPE(geo,0.001))
FROM GeoDbLand g
WHERE g.geo.Get_GType() = 3 OR g.geo.Get_GType() = 7;

SELECT SDO_AGGR_CENTROID(MDSYS.SDOAGGRTYPE(geo,0.001))
FROM GeoDbLand;


-- -----------------------
-- RUMLICHE BEZUGSSYSTEME
-- -----------------------

-- Ausgewhlte Koordinatensysteme abfragen:
SELECT srid, cs_name FROM MDSYS.CS_SRS WHERE srid IN (8307,82027,262148);

-- Gruppierung der Bezugssysteme:
SELECT COUNT(*), SUBSTR(wktext,1,6)
FROM MDSYS.CS_SRS
GROUP BY SUBSTR(wktext,1,6);

SELECT SRID, WKTEXT FROM MDSYS.CS_SRS WHERE SRID IN (8307,82027,262148);

-- Abfrage der Zusatztabellen:
SELECT * FROM MDSYS.SDO_ELLIPSOIDS WHERE name = 'WGS 84';
SELECT unit_name, conversion_factor FROM MDSYS.SDO_ANGLE_UNITS;
SELECT * FROM MDSYS.SDO_DATUMS;
SELECT * FROM MDSYS.SDO_PROJECTIONS;

-- Tabelle Gemeinden um Gebiet erweitern

-- Attribut entfernen:
ALTER TABLE Gemeinden DROP (centrum);
-- Attribut neu hinzufgen:
ALTER TABLE Gemeinden ADD centrum MDSYS.SDO_GEOMETRY;

-- Metadaten bekanntgeben:
INSERT INTO USER_SDO_GEOM_METADATA
VALUES ('GEMEINDEN','CENTRUM', 
        MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('Lnge',-180,180,100),
                            MDSYS.SDO_DIM_ELEMENT('Breite',-90,90,10)),
        8307);
COMMIT;

-- Attributswerte setzen:
UPDATE Gemeinden SET centrum=MDSYS.SDO_GEOMETRY(2001,8307,MDSYS.SDO_POINT_TYPE(9.7358,52.3803,NULL),NULL,NULL) WHERE gkz = 3241001;
UPDATE Gemeinden SET centrum=MDSYS.SDO_GEOMETRY(2001,8307,MDSYS.SDO_POINT_TYPE(8.2275,53.1375,NULL),NULL,NULL) WHERE gkz = 3403000;
UPDATE Gemeinden SET centrum=MDSYS.SDO_GEOMETRY(2001,8307,MDSYS.SDO_POINT_TYPE(7.9981,53.1844,NULL),NULL,NULL) WHERE gkz = 3451002;
UPDATE Gemeinden SET centrum=MDSYS.SDO_GEOMETRY(2001,8307,MDSYS.SDO_POINT_TYPE(7.9256,53.2581,NULL),NULL,NULL) WHERE gkz = 3451007;
UPDATE Gemeinden SET centrum=MDSYS.SDO_GEOMETRY(2001,8307,MDSYS.SDO_POINT_TYPE(7.2072,53.3686,NULL),NULL,NULL) WHERE gkz = 3402000;
UPDATE Gemeinden SET centrum=MDSYS.SDO_GEOMETRY(2001,8307,MDSYS.SDO_POINT_TYPE(8.1081,53.5397,NULL),NULL,NULL) WHERE gkz = 3405000;
COMMIT;

-- Puffer mit 1000m Abstand und Annherungstoleranz von 50m:
SELECT SDO_GEOM.SDO_BUFFER(centrum,1000,10,'arc_tolerance=50')
FROM Gemeinden
WHERE name = 'Oldenburg';

-- Puffer mit 1000m Abstand und Annherungstoleranz von 200m:
SELECT SDO_GEOM.SDO_BUFFER(centrum,1000,10,'arc_tolerance=200')
FROM Gemeinden
WHERE name = 'Oldenburg';

-- Distanz in km:
SELECT SDO_GEOM.SDO_DISTANCE(g1.centrum,m.diminfo,g2.centrum,m.diminfo,'unit=KM') 
FROM Gemeinden g1, Gemeinden g2, USER_SDO_GEOM_METADATA m
WHERE g1.name='Oldenburg' AND g2.name='Hannover' AND m.table_name='GEMEINDEN' AND m.column_name='CENTRUM';

-- Tabelle Gemeinden um Gebiet erweitern

-- Attribut entfernen:
ALTER TABLE Gemeinden DROP (gebiet);
-- Attribut neu hinzufgen:
ALTER TABLE Gemeinden ADD gebiet MDSYS.SDO_GEOMETRY;

-- Metadaten bekanntgeben:
INSERT INTO USER_SDO_GEOM_METADATA
VALUES ('GEMEINDEN','GEBIET',
         MDSYS.SDO_DIM_ARRAY(MDSYS.SDO_DIM_ELEMENT('Lnge',-180,180,10),
                             MDSYS.SDO_DIM_ELEMENT('Breite',-90,90,10)),
         8307);
COMMIT;

-- Attributswert setzen:
UPDATE Gemeinden SET gebiet=
MDSYS.SDO_GEOMETRY (2003 ,8307,NULL,MDSYS.SDO_ELEM_INFO_ARRAY(1,1003,1),
MDSYS.SDO_ORDINATE_ARRAY(
8.214289,53.087776, 8.23346, 53.089134, 8.277027,53.099326,
8.284276,53.121173, 8.296767,53.145781, 8.305773,53.148083,
8.307031,53.153128, 8.300405,53.162775, 8.299069,53.171417,
8.298319,53.192052, 8.279517,53.191205, 8.255955,53.194478,
8.24101, 53.201628, 8.226077,53.200909, 8.213507,53.204358,
8.201577,53.206405, 8.188254,53.19307,  8.173382,53.184449,
8.158582,53.177491, 8.164676,53.170631, 8.153453,53.165192,
8.152922,53.153215, 8.143793,53.136619, 8.164244,53.11119,
8.20185, 53.115248, 8.193622,53.097121, 8.204879,53.089616,
8.214289,53.087776 ))
WHERE gkz = 3403000;

UPDATE Gemeinden SET gebiet=
MDSYS.SDO_GEOMETRY (2003 ,8307,NULL,MDSYS.SDO_ELEM_INFO_ARRAY(1,1003,1),
MDSYS.SDO_ORDINATE_ARRAY(
9.820821,52.443919, 9.800375,52.450597, 9.773278,52.452986,
9.730943,52.429469, 9.723265,52.429253, 9.654279,52.433481,
9.621969,52.427492, 9.616331,52.407012, 9.609993,52.395092,
9.629717,52.401469, 9.640883,52.411954, 9.648996,52.399476,
9.655472,52.398622, 9.651906,52.384535, 9.660093,52.375006,
9.643668,52.36304,  9.646189,52.352631, 9.682835,52.340672,
9.670515,52.328613, 9.678468,52.314826, 9.717761,52.330883,
9.738732,52.337596, 9.764779,52.324283, 9.787924,52.322608,
9.811941,52.318664, 9.844539,52.306531, 9.862442,52.321224,
9.876362,52.335064, 9.87331, 52.350532, 9.894356,52.36427,
9.898358,52.400342, 9.90666, 52.411375, 9.888405,52.42307,
9.836918,52.421681, 9.828833,52.435279, 9.820821,52.443919 ))
WHERE gkz = 3241001;
COMMIT;

-- Distanz in km:
SELECT SDO_GEOM.SDO_DISTANCE(g1.gebiet,m.diminfo,g2.gebiet,m.diminfo,'unit=KM') 
FROM Gemeinden g1, Gemeinden g2, USER_SDO_GEOM_METADATA m
WHERE g1.name='Oldenburg' AND g2.name='Hannover' AND m.table_name='GEMEINDEN' AND m.column_name='GEBIET';


-- Flche und Umfang:
SELECT name, SDO_GEOM.SDO_AREA(gebiet,100,'unit=SQ_KM') AS flche,SDO_GEOM.SDO_LENGTH(gebiet,100,'unit=KM') AS umfang
FROM Gemeinden
WHERE gebiet IS NOT NULL;

-- Koordinatentransformationen

-- Koordinatentransformationen ausfhren:
SELECT SDO_CS.TRANSFORM (gebiet,82027) FROM Gemeinden WHERE gkz = 3403000;
SELECT SDO_CS.TRANSFORM (gebiet,'GK Zone 3 (DHDN)') FROM Gemeinden WHERE gkz = 3403000;

-- Puffer mit 1000m Abstand und Annherungstoleranz von 50m:
SELECT SDO_GEOM.SDO_BUFFER(SDO_CS.TRANSFORM (centrum,82027),1000,10)
FROM Gemeinden
WHERE name = 'Oldenburg';


-- ---------------------
-- LINEARE BEZUGSSYSTEME
-- ---------------------

-- Tabelle definieren:
CREATE TABLE Netzwerk (
  id     INTEGER,                        -- Identifikationsnummer
  name   VARCHAR(20),                    -- Objektname
  geo    MDSYS.SDO_GEOMETRY,             -- Objektgeometrie
  CONSTRAINT pk_netzwerk PRIMARY KEY(id) -- Primrschlssel
);

-- Metadaten einfgen:
INSERT INTO USER_SDO_GEOM_METADATA
VALUES ('NETZWERK','GEO', MDSYS.SDO_DIM_ARRAY(
            MDSYS.SDO_DIM_ELEMENT('X',0,20,0.001),
            MDSYS.SDO_DIM_ELEMENT('Y',0,15,0.001),
            MDSYS.SDO_DIM_ELEMENT('Abstand',0,25,0.001)),
        NULL);
COMMIT;

-- Daten einfgen:
INSERT INTO Netzwerk (id,name,geo)
VALUES (1,'Weg 1',MDSYS.SDO_GEOMETRY(3302,NULL,NULL,
                    MDSYS.SDO_ELEM_INFO_ARRAY(1,2,1),
                    MDSYS.SDO_ORDINATE_ARRAY(5,10,4.18,5,8,2.12,7,8,0)));

INSERT INTO Netzwerk (id,name,geo)
VALUES (2,'Weg 2',MDSYS.SDO_GEOMETRY(3302,NULL,NULL,
                    MDSYS.SDO_ELEM_INFO_ARRAY(1,2,1),
                    MDSYS.SDO_ORDINATE_ARRAY(5,5,NULL,10,10,NULL,15,5,NULL,15,10,NULL)));
INSERT INTO Netzwerk (id,name,geo)
VALUES (3,'Weg 3',MDSYS.SDO_GEOMETRY(3302,NULL,NULL,
                    MDSYS.SDO_ELEM_INFO_ARRAY(1,4,2,1,2,2,7,2,1),
                    MDSYS.SDO_ORDINATE_ARRAY(5,3,5, 6.414,3.586,NULL, 7,5,NULL, 15,3,16.3878)));
COMMIT;

-- Prozedur zur Ergnzung einer Geometrie um Abstandsangaben
-- id_param: id des Tupels in der Tabelle Netzwerk
CREATE OR REPLACE PROCEDURE berechneAbstand (id_param IN DECIMAL) IS
  geo_var MDSYS.SDO_GEOMETRY;
BEGIN
  SELECT geo INTO geo_var FROM Netzwerk WHERE id = id_param FOR UPDATE;
  SDO_LRS.DEFINE_GEOM_SEGMENT(geo_var);
  UPDATE Netzwerk SET geo=geo_var WHERE id = id_param;
  COMMIT;
END;
/

-- Ausfhren der Ergnzung:
EXECUTE berechneAbstand(2);
EXECUTE berechneAbstand(3);

-- berprfung des Ergebnisses:
SELECT name, geo FROM Netzwerk WHERE id > 1;

-- Abstnde ber Cursor berechnen:
CREATE OR REPLACE PROCEDURE berechneAlleAbstnde IS
  -- Deklaration Cursor und Variable
  CURSOR c IS SELECT id,geo,diminfo
              FROM Netzwerk n, USER_SDO_GEOM_METADATA m
              WHERE m.table_name='NETZWERK' AND m.column_name='GEO' AND n.id > 1;   -- DB-Cursor
  akt_ds c%ROWTYPE;                 -- aktuelle Antwort
BEGIN
  -- alle Antworten durchlaufen
  FOR akt_ds IN c LOOP
    SDO_LRS.DEFINE_GEOM_SEGMENT(akt_ds.geo,akt_ds.diminfo);
    UPDATE Netzwerk SET geo=akt_ds.geo WHERE id = akt_ds.id;
  END LOOP;
  COMMIT;
END;
/

CALL berechneAlleAbstnde();

-- Geometrien validieren:
SELECT name, SDO_LRS.VALIDATE_LRS_GEOMETRY(geo,m.diminfo)
FROM Netzwerk, USER_SDO_GEOM_METADATA m
WHERE m.table_name = 'NETZWERK' AND m.column_name = 'GEO';

-- Abstnde berechnen:
SELECT name, SDO_LRS.MEASURE_RANGE(geo)
FROM Netzwerk;

SELECT name, SDO_LRS.MEASURE_RANGE(geo,m.diminfo)
FROM Netzwerk, USER_SDO_GEOM_METADATA m
WHERE m.table_name = 'NETZWERK' AND m.column_name = 'GEO';

-- Koordinaten berechnen:
SELECT name, SDO_LRS.LOCATE_PT(geo,1.5)
FROM Netzwerk
WHERE id <> 3;

-- Koordinaten mit Offset berechnen:
SELECT name,SDO_LRS.LOCATE_PT(geo,1.5,1)
FROM Netzwerk
WHERE id <> 3;

SELECT name,SDO_LRS.LOCATE_PT(geo,1.5,-1)
FROM Netzwerk
WHERE id <> 3;

-- Abstand bestimmen:
SELECT SDO_LRS.FIND_MEASURE(geo,MDSYS.SDO_GEOMETRY(3001, NULL,NULL,
                                  SDO_ELEM_INFO_ARRAY(1,1,1),
                                  SDO_ORDINATE_ARRAY(7,7, NULL)) )
FROM Netzwerk
WHERE id = 2;

SELECT SDO_LRS.FIND_MEASURE(geo,MDSYS.SDO_GEOMETRY(3001, NULL,NULL,
                                  SDO_ELEM_INFO_ARRAY(1,1,1),
                                  SDO_ORDINATE_ARRAY(8,9, NULL)) )
FROM Netzwerk
WHERE id = 2;

-- Versatz und Projektionspunkt bestimmen:
SELECT SDO_LRS.FIND_OFFSET(geo,SDO_GEOMETRY(3001, NULL,NULL,
                                 SDO_ELEM_INFO_ARRAY(1,1,1),
                                 SDO_ORDINATE_ARRAY(8,9, NULL)), 0.001)
FROM Netzwerk
WHERE id = 2;

SELECT SDO_LRS.PROJECT_PT(geo,SDO_GEOMETRY(3001, NULL,NULL,
                                 SDO_ELEM_INFO_ARRAY(1,1,1),
                                 SDO_ORDINATE_ARRAY(8,9, NULL)), 0.001)
FROM Netzwerk
WHERE id = 2;


-- -------------------
-- IMPORT von GEODATEN
-- -------------------

--- Import von Punktdaten

-- Temporre Tabelle anlegen:
CREATE TABLE GeoTemp (
  id     DECIMAL(8),                     -- Identifikationsnummer
  geo    MDSYS.SDO_GEOMETRY,             -- Objektgeometrie
  CONSTRAINT pk_geotemp PRIMARY KEY(id)  -- Primrschlssel
);

-- Aufruf des SQL*Loaders (von der Kommandozeile aus):
SQLLDR USERID=<Benutzer>@<Dienstname> CONTROL=centrum.ctl LOG=centrum.log ERRORS=50

-- berfhrung der Daten:
UPDATE Gemeinden SET centrum = (SELECT geo FROM GeoTemp WHERE id = gkz);
COMMIT;

-- Aufruf des SQL*Loaders (von der Kommandozeile aus):
SQLLDR USERID=<Benutzer>@<Dienstname> CONTROL=gemeinden.ctl LOG=gemeinden.log ERRORS=50

-- Daten berprfen:
SELECT gkz, name, centrum FROM Gemeinden;

--- Import der Vorwahlgebiete

-- Tabelle anlegen:
CREATE TABLE Ortsnetze (
  vorwahl VARCHAR(6),           -- Vorwahlnummer
  name VARCHAR(80),             -- Name des Ortsnetzes
  gebiet  MDSYS.SDO_GEOMETRY,   -- Gebiet des Ortsnetzes
  CONSTRAINT pk_vorwahlgebiete PRIMARY KEY(vorwahl)
);

-- Metadaten einfgen:
INSERT INTO USER_SDO_GEOM_METADATA
VALUES ('ORTSNETZE','GEBIET', MDSYS.SDO_DIM_ARRAY(
  MDSYS.SDO_DIM_ELEMENT('X',-180,180,1),
  MDSYS.SDO_DIM_ELEMENT('Y',-90,90,1)
) , 8307);
COMMIT;

-- Aufruf des SQL*Loaders (von der Kommandozeile aus):
SQLLDR USERID=<Benutzer>@<Dienstname> CONTROL=vorwahl.ctl LOG=vorwahl.log ERRORS=50

-- Daten berprfen:
SELECT * FROM Ortsnetze;
